#include "Controller-component.hpp"
#include <rtt/Component.hpp>
#include <rtt/scripting/Scripting.hpp>
#include <rtt/scripting/ProgramInterface.hpp>
#include <rtt/scripting/ScriptingService.hpp>


/*
 * Using this macro, only one component may live
 * in one library *and* you may *not* link this library
 * with another component library. Use 
 * ORO_CREATE_COMPONENT_TYPE()
 * ORO_LIST_COMPONENT_TYPE(Controller)
 * In case you want to link with another library that
 * already contains components.
 *
 * If you have put your component class
 * in a namespace, don't forget to add it here too:
 */

using namespace boost;

ORO_CREATE_COMPONENT(Controller)

Controller::Controller(const string & name) :
		TaskContext(name), sensorValues("SensorValues"), steeringSignals(
				"SteeringSignals", 30), gain("Gain", "The proportional gain.") {

	this->properties()->addProperty("Gain", gain).doc("Gain of our controller");

	this->attributes()->addAttribute("temporary", tmp);

	this->ports()->addPort("SensorValues", sensorValues).doc(
			"1D measurement of this controller.");

	this->ports()->addPort("SteeringSignals", steeringSignals).doc(
			"1D action of this controller.");

	this->addOperation("nextPosition", &Controller::nextPosition, this,
			OwnThread).doc("Returns a reference position!");

	//this->getProvider<Scripting>("scripting");


}

bool Controller::configureHook() {
	return 1;
}

bool Controller::startHook() {
	scripting::ScriptingService::shared_ptr ss = boost::dynamic_pointer_cast<scripting::ScriptingService>(this->provides()->getService("scripting"));
	scripting::ProgramInterfacePtr pi = ss->getProgram("ControllerAction");
	if(pi)
		return pi->start();
	return false;

}

void Controller::updateHook() {

}

void Controller::stopHook() {
	scripting::ScriptingService::shared_ptr ss = boost::dynamic_pointer_cast<scripting::ScriptingService>(this->provides()->getService("scripting"));
	scripting::ProgramInterfacePtr pi = ss->getProgram("ControllerAction");
		if(pi)
			pi->stop();
}

void Controller::cleanupHook() {
}

double Controller::nextPosition() {
	double w = 1.0 * 2.0 * 3.14;
	tmp = tmp + 0.001;
	return 5 * sin(w * tmp);
}

